//---------------------------------------------------------------------------
#include "stm32f4xx.h"
#include <stdio.h>
//------------------------------------------------------------------------------
#define DISPLAY_MODE  0x40  //Or Keyscan mode 0x42
#define DISPLAY_CTRL  0x87
#define DISPLAY_REG0  0xC0
#define DISPLAY_REG1  0xC1
#define DISPLAY_REG2  0xC2
#define DISPLAY_REG3  0xC3
#define DISPLAY_REG4  0xC4
#define DISPLAY_REG5  0xC5
#define DECIMAL_POINT 0x80
//------------------------------------------------------------------------------
void TM1637_init(char level);
void TM1637_display(signed short number);
void TM1637_setBrightness(char brightness);
void TM1637_write(char text[]);
void TM1637_command(char command);
void TM1637_initSend(char command);
char getDigit(char pos,char asciiChar);
//------------------------------------------------------------------------------
void ADC_channel(unsigned short channel);
void ADC_init(void);
int ADC_read(void);
float getTemperature(uint32_t adc_value);
//------------------------------------------------------------------------------
void HSI_init(void);
void delay_ms(int n);
void delay_us (int time); 
//------------------------------------------------------------------------------
int result;
signed short tempr;

int main()
{	  
	HSI_init();
	TM1637_init(8);       // init with brightness value. Valid values: 0 - 8.
	ADC_channel(0);
	ADC_init();
	
	while(1)
	{
		result = ADC_read();
		tempr = (signed short) getTemperature(result) ;
		TM1637_display(tempr * 10);
		delay_ms(2000);
	}
}
//---------------------------------------------------------------------------
void HSI_init(void)  //Set Clock to 16Mhz
{
	// Set the HSION bit and wait for HSIRDY
	RCC->CR |= 1ul << 0;
	while(!(RCC->CR & 2ul)){};

  // System clock switch to HSI		
  RCC->CFGR &= ~(3ul << 0);   // HSI oscillator selected as system clock
	RCC->CFGR &= ~(7ul << 27);  // MCO2 Prescaler. NO DIVISION selected 
	RCC->CFGR &= ~(3ul << 30);  // System clock (SYSCLK) selected
}
//---------------------------------------------------------------------------
void ADC_channel(unsigned short channel)
{
	//Setup pin PC0 for analog input
	RCC->AHB1ENR |= 4; //Enable GPIOC clock
	GPIOC->MODER |= (3 << 2 * channel); //PC0 selected analog
}
//-----------------------------------------------------------------------------
void ADC_init(void)
{	
	//Setup ADC1
	RCC->APB2ENR |= RCC_APB2ENR_ADC1EN;   // Enable ADC1 clock
	
	//External 
	ADC->CCR |= ADC_CCR_TSVREFE;          // Enable temperature sensor (Temperature Sensor and VREFINT Enable)
	ADC1->SQR3 = 10;                      // Channel 10 for external temperature sensor
	ADC1->SMPR1 |= 0;  										// Sampling time of 3 cycles

	// Enable ADC1
	ADC1->CR2 |= ADC_CR2_ADON;
}
//---------------------------------------------------------------------------
int ADC_read(void)
{
	int result;
	
	ADC1->CR2 |= 0x40000000; //Start conversion
	while(!(ADC1->SR & 2));  //Wait for conversion to complete
	result = ADC1->DR;       //Read conversion
	
	return result;
}
//---------------------------------------------------------------------------
float getTemperature(uint32_t adc_value)
{
	float tempr;
	
	tempr = (double)adc_value / 4095 * 330;
  
	return tempr;
}
//---------------------------------------------------------------------------
void delay_ms(int n) 
{
  int i;
  for(; n > 0; n--)
    for(i = 0; i < 3195; i++);
}
//---------------------------------------------------------------------------
void delay_us(int time)
{
	for(int i = 0; i < time; i++)
		for(int j = 0; j < 7; j++);
}
//------------------------------------------------------------------------------
void twoWire_init()
{	
	//GPIOA Clock Enable
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;  
	
	//Configure PA0 
	GPIOA->MODER &= ~0x00000003;
	GPIOA->MODER |= 0x00000001;       	//SCK is Output
	
	//Configure PA1
	GPIOA->MODER &= ~0x00000000C;
	GPIOA->MODER |= 0x00000004;					//SDA is Output
	
	//PA0 and PA1 initialsed to 0
	GPIOA->BSRR = GPIO_BSRR_BR0;				//SCK = 0 
	GPIOA->BSRR = GPIO_BSRR_BR1;				//SDA = 0 
}
//------------------------------------------------------------------------------
void twoWire_start()
{	
	GPIOA->BSRR = GPIO_BSRR_BS0;				//SCK = 1 
	GPIOA->BSRR = GPIO_BSRR_BS1;				//SDA = 1
	delay_us(2);
	GPIOA->BSRR = GPIO_BSRR_BR1;				//SDA = 0 	
}
//------------------------------------------------------------------------------
void twoWire_stop()
{	
	GPIOA->BSRR = GPIO_BSRR_BR0;				//SCK = 0 
	delay_us(2);
	GPIOA->BSRR = GPIO_BSRR_BR1;				//SDA = 0
	delay_us(2);
	GPIOA->BSRR = GPIO_BSRR_BS0;				//SCK = 1
	delay_us(2);
	GPIOA->BSRR = GPIO_BSRR_BS1;				//SDA = 1
}
//------------------------------------------------------------------------------
void twoWire_ack()
{	
	GPIOA->BSRR = GPIO_BSRR_BR0;				//SCK = 0 
	delay_us(5);
	GPIOA->BSRR = GPIO_BSRR_BS0;				//SCK = 1
	delay_us(2);
	GPIOA->BSRR = GPIO_BSRR_BR0;				//SCK = 0 
}
//------------------------------------------------------------------------------
void twoWire_write(char data)
{
  for(char tx = 0 ; tx < 8 ; tx++)
  {		
		GPIOA->BSRR = GPIO_BSRR_BR0;			//SCK = 0
		GPIOA->BSRR = ((data >> tx) & 0x01) ? GPIO_BSRR_BS1 : GPIO_BSRR_BR1 ; //LSB first (Real i2c sends MSB first)
		GPIOA->BSRR = GPIO_BSRR_BS0;			//SCK = 1
		
		delay_us(6);
  }
}
//------------------------------------------------------------------------------
//Common Anode             0,    1,    2,    3,    4,    5,    6,    7,    8,   9
const char digits[] = { 0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x6F };
//------------------------------------------------------------------------------
void TM1637_init(char level)
{
  twoWire_init();    
  TM1637_setBrightness(level); //Valid brightness values: 0 - 8. 0 = display off.
}
//------------------------------------------------------------------------------
void TM1637_display(signed short number)
{    
	char strNum[10];
	sprintf(strNum,"%4d",number);
	
	TM1637_command(DISPLAY_MODE);    
	TM1637_write(strNum);
}
//------------------------------------------------------------------------------
void TM1637_setBrightness(char level)
{    
  TM1637_command(DISPLAY_CTRL + level); //Valid brightness values: 0 - 8. 0 = display off.
}
//------------------------------------------------------------------------------
void TM1637_write(char text[])
{
	TM1637_initSend(DISPLAY_REG0);
    
	char pos = 0;
  while(*text)
  {        
		twoWire_write( getDigit(pos++, *text++) );
		twoWire_ack();
  }
  twoWire_stop();
}
//------------------------------------------------------------------------------
void TM1637_command(char command)
{
  TM1637_initSend(command);
  twoWire_stop();
}
//------------------------------------------------------------------------------
void TM1637_initSend(char command)
{
  twoWire_start();
  twoWire_write(command);
  twoWire_ack();
}
//------------------------------------------------------------------------------
char getDigit(char pos,char asciiChar)
{
  char digit = 0;
    
	if(asciiChar >= '0' && asciiChar <= '9')
	{
			char i = asciiChar - '0'; //Get index 0 - 9
			digit = (pos == 2) ? (digits[i] + DECIMAL_POINT) : digits[i];
	}
	else
			if(asciiChar == '-')
			{
					digit = (pos == 2) ? (digits[0]  + DECIMAL_POINT) : 0x40;  //0x40 - minus symbol
			}
			else
					if(asciiChar == ' ')
					{
							digit = (pos == 2) ? (digits[0] + DECIMAL_POINT) : 0x00; //0x00 - blank. used for leading zero suppression
					}
	
	return digit;
}
//------------------------------------------------------------------------------
